@@ -27,6 +27,7 @@
 
 #include <gtk/gtkobject.h>
 #include <libgnomevfs/gnome-vfs-types.h>
+#include <libgnomevfs/gnome-vfs-volume.h>
 #include <libnautilus-private/nautilus-file-attributes.h>
 
 /* NautilusFile is an object used to represent a single element of a
@@ -77,6 +78,8 @@ typedef enum {
 
 typedef void (*NautilusFileCallback)          (NautilusFile  *file,
 				               gpointer       callback_data);
+typedef void (*NautilusFileListCallback)      (GList         *file_list,
+				               gpointer       callback_data);
 typedef void (*NautilusFileOperationCallback) (NautilusFile  *file,
 					       GnomeVFSResult result,
 					       gpointer       callback_data);
@@ -134,9 +137,11 @@ char *                  nautilus_file_get_parent_uri                    (Nautilu
 char *                  nautilus_file_get_parent_uri_for_display        (NautilusFile                   *file);
 GnomeVFSFileSize        nautilus_file_get_size                          (NautilusFile                   *file);
 GnomeVFSFileType        nautilus_file_get_file_type                     (NautilusFile                   *file);
+char *                  nautilus_file_get_guessed_mime_type             (NautilusFile                   *file);
 char *                  nautilus_file_get_mime_type                     (NautilusFile                   *file);
 gboolean                nautilus_file_is_mime_type                      (NautilusFile                   *file,
 									 const char                     *mime_type);
+gboolean                nautilus_file_needs_slow_mime_type              (NautilusFile                   *file);
 gboolean                nautilus_file_is_symbolic_link                  (NautilusFile                   *file);
 char *                  nautilus_file_get_volume_free_space             (NautilusFile                   *file);
 char *                  nautilus_file_get_volume_name                   (NautilusFile                   *file);
@@ -148,6 +153,8 @@ gboolean                nautilus_file_is_executable                     (Nautilu
 gboolean                nautilus_file_is_directory                      (NautilusFile                   *file);
 gboolean                nautilus_file_is_in_trash                       (NautilusFile                   *file);
 gboolean                nautilus_file_is_in_desktop                     (NautilusFile                   *file);
+gboolean		nautilus_file_is_home				(NautilusFile                   *file);
+gboolean                nautilus_file_is_desktop_directory              (NautilusFile                   *file);
 GnomeVFSResult          nautilus_file_get_file_info_result              (NautilusFile                   *file);
 gboolean                nautilus_file_get_directory_item_count          (NautilusFile                   *file,
 									 guint                          *count,
@@ -157,7 +164,8 @@ NautilusRequestStatus   nautilus_file_get_deep_counts                   (Nautilu
 									 guint                          *directory_count,
 									 guint                          *file_count,
 									 guint                          *unreadable_directory_count,
-									 GnomeVFSFileSize               *total_size);
+									 GnomeVFSFileSize               *total_size,
+									 gboolean                        force);
 gboolean                nautilus_file_should_show_directory_item_count  (NautilusFile                   *file);
 GList *                 nautilus_file_get_keywords                      (NautilusFile                   *file);
 void                    nautilus_file_set_keywords                      (NautilusFile                   *file,
@@ -180,7 +188,7 @@ gboolean                nautilus_file_can_set_group                     (Nautilu
 char *                  nautilus_file_get_owner_name                    (NautilusFile                   *file);
 char *                  nautilus_file_get_group_name                    (NautilusFile                   *file);
 GList *                 nautilus_get_user_names                         (void);
-GList *                 nautilus_get_group_names                        (void);
+GList *                 nautilus_get_all_group_names                    (void);
 GList *                 nautilus_file_get_settable_group_names          (NautilusFile                   *file);
 
 /* "Capabilities". */
@@ -189,6 +197,12 @@ gboolean                nautilus_file_can_write                         (Nautilu
 gboolean                nautilus_file_can_execute                       (NautilusFile                   *file);
 gboolean                nautilus_file_can_rename                        (NautilusFile                   *file);
 
+/* Volumes. */
+gboolean                nautilus_file_has_volume                        (NautilusFile                   *file);
+gboolean                nautilus_file_has_drive                         (NautilusFile                   *file);
+GnomeVFSVolume *        nautilus_file_get_volume                        (NautilusFile                   *file);
+GnomeVFSDrive *         nautilus_file_get_drive                         (NautilusFile                   *file);
+
 /* Basic operations for file objects. */
 void                    nautilus_file_set_owner                         (NautilusFile                   *file,
 									 const char                     *user_name_or_id,
@@ -279,6 +293,11 @@ int                     nautilus_file_compare_for_sort                  (Nautilu
 									 NautilusFileSortType            sort_type,
 									 gboolean			 directories_first,
 									 gboolean		  	 reversed);
+int                     nautilus_file_compare_for_sort_by_attribute     (NautilusFile                   *file_1,
+									 NautilusFile                   *file_2,
+									 const char                     *attribute,
+									 gboolean                        directories_first,
+									 gboolean                        reversed);
 int                     nautilus_file_compare_display_name              (NautilusFile                   *file_1,
 									 const char                     *pattern);
 
@@ -304,6 +323,10 @@ char *                  nautilus_file_get_drop_target_uri               (Nautilu
 char *                  nautilus_file_get_custom_icon                   (NautilusFile                   *file);
 
 
+gboolean                nautilus_file_has_open_window                   (NautilusFile                   *file);
+void                    nautilus_file_set_has_open_window               (NautilusFile                   *file,
+									 gboolean                        has_open_window);
+
 /* Thumbnailing handling */
 gboolean                nautilus_file_is_thumbnailing                   (NautilusFile                   *file);
 
@@ -315,6 +338,10 @@ void                    nautilus_file_list_unref                        (GList
 void                    nautilus_file_list_free                         (GList                          *file_list);
 GList *                 nautilus_file_list_copy                         (GList                          *file_list);
 GList *			nautilus_file_list_sort_by_display_name		(GList				*file_list);
+void                    nautilus_file_list_call_when_ready              (GList                          *file_list,
+									 NautilusFileAttributes          attributes,
+									 NautilusFileListCallback        callback,
+									 gpointer                        callback_data);
 
 /* Debugging */
 void                    nautilus_file_dump                              (NautilusFile                   *file);
