@@ -67,9 +67,20 @@
 #include <libnautilus-private/nautilus-tree-view-drag-dest.h>
 #include <libnautilus-private/nautilus-icon-factory.h>
 #include <libnautilus-private/nautilus-cell-renderer-pixbuf-emblem.h>
+#include <libnautilus-private/nautilus-sidebar-provider.h>
+#include <libnautilus-private/nautilus-module.h>
+
+typedef struct {
+        GObject parent;
+} FMTreeViewProvider;
+
+typedef struct {
+        GObjectClass parent;
+} FMTreeViewProviderClass;
+
 
 struct FMTreeViewDetails {
-	GtkWidget *scrolled_window;
+	NautilusWindowInfo *window;
 	GtkTreeView *tree_widget;
 	GtkTreeModelSort *sort_model;
 	FMTreeModel *child_model;
@@ -116,8 +127,18 @@ static const GtkTargetEntry clipboard_targets[] = {
 	{ "x-special/gnome-copied-files", 0, GNOME_COPIED_FILES },
 };
 
-BONOBO_CLASS_BOILERPLATE (FMTreeView, fm_tree_view,
-			  NautilusView, NAUTILUS_TYPE_VIEW)
+static void  fm_tree_view_iface_init        (NautilusSidebarIface         *iface);
+static void  sidebar_provider_iface_init    (NautilusSidebarProviderIface *iface);
+static GType fm_tree_view_provider_get_type (void);
+
+G_DEFINE_TYPE_WITH_CODE (FMTreeView, fm_tree_view, GTK_TYPE_SCROLLED_WINDOW,
+			 G_IMPLEMENT_INTERFACE (NAUTILUS_TYPE_SIDEBAR,
+						fm_tree_view_iface_init));
+#define parent_class fm_tree_view_parent_class
+
+G_DEFINE_TYPE_WITH_CODE (FMTreeViewProvider, fm_tree_view_provider, G_TYPE_OBJECT,
+			 G_IMPLEMENT_INTERFACE (NAUTILUS_TYPE_SIDEBAR_PROVIDER,
+						sidebar_provider_iface_init));
 
 static gboolean
 show_iter_for_file (FMTreeView *view, NautilusFile *file, GtkTreeIter *iter)
@@ -139,7 +160,10 @@ show_iter_for_file (FMTreeView *view, NautilusFile *file, GtkTreeIter *iter)
 		if (gtk_tree_model_get_iter (model, &cur_iter, path)) {
 			if (fm_tree_model_file_get_iter (view->details->child_model,
 							       iter, file, &cur_iter)) {
+				gtk_tree_path_free (path);
 				return TRUE;
+			} else {
+				gtk_tree_path_free (path);
 			}
 		}
 	}
@@ -211,7 +235,9 @@ show_selection_idle_callback (gpointer callback_data)
 		(view->details->sort_model, path);
 	gtk_tree_path_free (path);
 	gtk_tree_view_set_cursor (view->details->tree_widget, sort_path, NULL, FALSE);
-	gtk_tree_view_scroll_to_cell (view->details->tree_widget, sort_path, NULL, FALSE, 0, 0);
+	if (GTK_WIDGET_REALIZED (view->details->tree_widget)) {
+		gtk_tree_view_scroll_to_cell (view->details->tree_widget, sort_path, NULL, FALSE, 0, 0);
+	}
 	gtk_tree_path_free (sort_path);
 
 	nautilus_file_unref (file);
@@ -227,6 +253,16 @@ schedule_show_selection (FMTreeView *view)
 	}
 }
 
+static void
+schedule_select_and_show_location (FMTreeView *view, char *location)
+{
+	if (view->details->selection_location != NULL) {
+		g_free (view->details->selection_location);
+	}
+	view->details->selection_location = g_strdup (location);
+	schedule_show_selection (view);
+}
+
 static void
 row_loaded_callback (GtkTreeModel     *tree_model,
 		     GtkTreeIter      *iter,
@@ -292,7 +328,7 @@ got_activation_uri_callback (NautilusFile *file, gpointer callback_data)
         char *uri, *file_uri;
         FMTreeView *view;
 	GdkScreen *screen;
-	Nautilus_ViewFrame_OpenMode mode;
+	NautilusWindowOpenMode mode;
 	
         view = FM_TREE_VIEW (callback_data);
 
@@ -300,7 +336,7 @@ got_activation_uri_callback (NautilusFile *file, gpointer callback_data)
 
         g_assert (file == view->details->activation_file);
         
-        mode = view->details->activation_in_new_window ? Nautilus_ViewFrame_OPEN_IN_NAVIGATION : Nautilus_ViewFrame_OPEN_ACCORDING_TO_MODE;
+        mode = view->details->activation_in_new_window ? NAUTILUS_WINDOW_OPEN_IN_NAVIGATION : NAUTILUS_WINDOW_OPEN_ACCORDING_TO_MODE;
 
 	/* FIXME: reenable && !eel_uris_match_ignore_fragments (view->details->current_main_view_uri, uri) */
 
@@ -327,8 +363,8 @@ got_activation_uri_callback (NautilusFile *file, gpointer callback_data)
 
 		/* Non-local executables don't get launched. They act like non-executables. */
 		if (file_uri == NULL) {
-			nautilus_view_open_location
-				(NAUTILUS_VIEW (view), 
+			nautilus_window_info_open_location
+				(view->details->window, 
 				 uri, 
 				 mode,
 				 0,
@@ -345,8 +381,8 @@ got_activation_uri_callback (NautilusFile *file, gpointer callback_data)
 				g_free (view->details->selection_location);
 			}
 			view->details->selection_location = g_strdup (uri);
-			nautilus_view_open_location
-				(NAUTILUS_VIEW (view), 
+			nautilus_window_info_open_location
+				(view->details->window, 
 				 uri,
 				 mode,
 				 0,
@@ -451,11 +487,12 @@ compare_rows (GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b, gpointer call
 	NautilusFile *file_a, *file_b;
 	int result;
 
+	/* Dummy rows are always first */
 	if (a->user_data == NULL) {
 		return -1;
 	}
 	else if (b->user_data == NULL) {
-		return -1;
+		return 1;
 	}
 
 	/* don't sort root nodes */
@@ -514,7 +551,7 @@ static void
 move_copy_items_callback (NautilusTreeViewDragDest *dest,
 			  const GList *item_uris,
 			  const char *target_uri,
-			  guint action,
+			  GdkDragAction action,
 			  int x,
 			  int y,
 			  gpointer user_data)
@@ -862,17 +899,17 @@ copy_or_cut_files (FMTreeView *view,
 	name = nautilus_file_get_display_name (view->details->popup_file);
 	if (cut) {
 		status_string = g_strdup_printf (_("\"%s\" will be moved "
-						   "if you select the Paste Files command"),
+						   "if you select the Paste command"),
 						 name);
 	} else {
 		status_string = g_strdup_printf (_("\"%s\" will be copied "
-						   "if you select the Paste Files command"),
+						   "if you select the Paste command"),
 						 name);
 	}
 	g_free (name);
 	
-	nautilus_view_report_status (NAUTILUS_VIEW (view),
-				     status_string);
+	nautilus_window_info_set_status (view->details->window,
+					 status_string);
 	g_free (status_string);
 }
 
@@ -896,15 +933,15 @@ convert_lines_to_str_list (char **lines, gboolean *cut)
 	int i;
 	GList *result;
 
+	*cut = FALSE;
+
 	if (lines[0] == NULL) {
 		return NULL;
 	}
 
 	if (strcmp (lines[0], "cut") == 0) {
 		*cut = TRUE;
-	} else if (strcmp (lines[0], "copy") == 0) {
-		*cut = FALSE;
-	} else {
+	} else if (strcmp (lines[0], "copy") != 0) {
 		return NULL;
 	}
 
@@ -923,7 +960,8 @@ paste_clipboard_data (FMTreeView *view,
 	char **lines;
 	gboolean cut;
 	GList *item_uris;
-	
+
+	cut = FALSE;
 	if (selection_data->type != copied_files_atom
 	    || selection_data->length <= 0) {
 		item_uris = NULL;
@@ -940,8 +978,8 @@ paste_clipboard_data (FMTreeView *view,
 	}
 
 	if (item_uris == NULL|| destination_uri == NULL) {
-		nautilus_view_report_status (NAUTILUS_VIEW (view),
-					     _("There is nothing on the clipboard to paste."));
+		nautilus_window_info_set_status (view->details->window,
+						 _("There is nothing on the clipboard to paste."));
 	} else {
 		nautilus_file_operations_copy_move
 			(item_uris, NULL, destination_uri,
@@ -1065,7 +1103,7 @@ create_popup_menu (FMTreeView *view)
 	menu_image = gtk_image_new_from_stock (GTK_STOCK_OPEN,
 					       GTK_ICON_SIZE_MENU);
 	gtk_widget_show (menu_image);
-	menu_item = gtk_image_menu_item_new_with_label (_("Open"));
+	menu_item = gtk_image_menu_item_new_with_mnemonic (_("_Open"));
 	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menu_item),
 				       menu_image);
 	g_signal_connect (menu_item, "activate",
@@ -1102,12 +1140,7 @@ create_popup_menu (FMTreeView *view)
 	gtk_menu_shell_append (GTK_MENU_SHELL (popup), separator_item);
 	
 	/* add the "cut folder" menu item */
-	menu_image = gtk_image_new_from_stock (GTK_STOCK_CUT,
-					       GTK_ICON_SIZE_MENU);
-	gtk_widget_show (menu_image);
-	menu_item = gtk_image_menu_item_new_with_label (_("Cut Folder"));
-	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menu_item),
-				       menu_image);
+	menu_item = gtk_image_menu_item_new_from_stock (GTK_STOCK_CUT, NULL);
 	g_signal_connect (menu_item, "activate",
 			  G_CALLBACK (fm_tree_view_cut_cb),
 			  view);
@@ -1116,12 +1149,7 @@ create_popup_menu (FMTreeView *view)
 	view->details->popup_cut = menu_item;
 	
 	/* add the "copy folder" menu item */
-	menu_image = gtk_image_new_from_stock (GTK_STOCK_COPY,
-					       GTK_ICON_SIZE_MENU);
-	gtk_widget_show (menu_image);
-	menu_item = gtk_image_menu_item_new_with_label (_("Copy Folder"));
-	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menu_item),
-				       menu_image);
+	menu_item = gtk_image_menu_item_new_from_stock (GTK_STOCK_COPY, NULL);
 	g_signal_connect (menu_item, "activate",
 			  G_CALLBACK (fm_tree_view_copy_cb),
 			  view);
@@ -1133,7 +1161,7 @@ create_popup_menu (FMTreeView *view)
 	menu_image = gtk_image_new_from_stock (GTK_STOCK_PASTE,
 					       GTK_ICON_SIZE_MENU);
 	gtk_widget_show (menu_image);
-	menu_item = gtk_image_menu_item_new_with_label (_("Paste Files into Folder"));
+	menu_item = gtk_image_menu_item_new_with_mnemonic (_("_Paste Into Folder"));
 	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menu_item),
 				       menu_image);
 	g_signal_connect (menu_item, "activate",
@@ -1166,12 +1194,7 @@ create_popup_menu (FMTreeView *view)
 	gtk_menu_shell_append (GTK_MENU_SHELL (popup), separator_item);
 	
 	/* add the "properties" menu item */
-	menu_image = gtk_image_new_from_stock (GTK_STOCK_PROPERTIES,
-					       GTK_ICON_SIZE_MENU);
-	gtk_widget_show (menu_image);
-	menu_item = gtk_image_menu_item_new_with_label (_("Properties"));
-	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menu_item),
-				       menu_image);
+	menu_item = gtk_image_menu_item_new_from_stock (GTK_STOCK_PROPERTIES, NULL);
 	g_signal_connect (menu_item, "activate",
 			  G_CALLBACK (fm_tree_view_properties_cb),
 			  view);
@@ -1205,6 +1228,7 @@ create_tree (FMTreeView *view)
 	GnomeVFSVolumeMonitor *volume_monitor;
 	char *home_uri;
 	GList *volumes, *l;
+	char *location;
 	
 	view->details->child_model = fm_tree_model_new ();
 	view->details->sort_model = GTK_TREE_MODEL_SORT
@@ -1219,7 +1243,7 @@ create_tree (FMTreeView *view)
 	home_uri = gnome_vfs_get_uri_from_local_path (g_get_home_dir ());
 	fm_tree_model_add_root_uri (view->details->child_model, home_uri, _("Home Folder"), "gnome-home", NULL);
 	g_free (home_uri);
-	fm_tree_model_add_root_uri (view->details->child_model, "file:///", _("Filesystem"), "gnome-fs-directory", NULL);
+	fm_tree_model_add_root_uri (view->details->child_model, "file:///", _("File System"), "gnome-fs-directory", NULL);
 #ifdef NOT_YET_USABLE
 	fm_tree_model_add_root_uri (view->details->child_model, "network:///", _("Network Neighbourhood"), "gnome-fs-network", NULL);
 #endif
@@ -1283,7 +1307,7 @@ create_tree (FMTreeView *view)
 
 	gtk_widget_show (GTK_WIDGET (view->details->tree_widget));
 
-	gtk_container_add (GTK_CONTAINER (view->details->scrolled_window),
+	gtk_container_add (GTK_CONTAINER (view),
 			   GTK_WIDGET (view->details->tree_widget));
 
 	g_signal_connect_object (gtk_tree_view_get_selection (GTK_TREE_VIEW (view->details->tree_widget)), "changed",
@@ -1297,28 +1321,30 @@ create_tree (FMTreeView *view)
 			  "button_press_event", G_CALLBACK (button_pressed_callback),
 			  view);
 
-	schedule_show_selection (view);
+	location = nautilus_window_info_get_current_location (view->details->window);
+	schedule_select_and_show_location (view, location);
+	g_free (location);
 }
 
 static void
 update_filtering_from_preferences (FMTreeView *view)
 {
-	Nautilus_ShowHiddenFilesMode mode;
+	NautilusWindowShowHiddenFilesMode mode;
 	
 	if (view->details->child_model == NULL) {
 		return;
 	}
 
-	mode = nautilus_view_get_show_hidden_files_mode (NAUTILUS_VIEW (view));
+	mode = nautilus_window_info_get_hidden_files_mode (view->details->window);
 
-	if (mode == Nautilus_SHOW_HIDDEN_FILES_DEFAULT) {
+	if (mode == NAUTILUS_WINDOW_SHOW_HIDDEN_FILES_DEFAULT) {
 		fm_tree_model_set_show_hidden_files
 			(view->details->child_model,
 			 eel_preferences_get_boolean (NAUTILUS_PREFERENCES_SHOW_HIDDEN_FILES));
 	} else {
 		fm_tree_model_set_show_hidden_files
 			(view->details->child_model,
-			 mode == Nautilus_SHOW_HIDDEN_FILES_ENABLE);
+			 mode == NAUTILUS_WINDOW_SHOW_HIDDEN_FILES_ENABLE);
 	}
 	fm_tree_model_set_show_backup_files
 		(view->details->child_model,
@@ -1329,13 +1355,15 @@ update_filtering_from_preferences (FMTreeView *view)
 }
 
 static void
-tree_activate_callback (BonoboControl *control, gboolean activating, gpointer user_data)
+parent_set_callback (GtkWidget        *widget,
+		     GtkWidget        *previous_parent,
+		     gpointer          callback_data)
 {
 	FMTreeView *view;
 
-	view = FM_TREE_VIEW (user_data);
+	view = FM_TREE_VIEW (callback_data);
 
-	if (activating && view->details->tree_widget == NULL) {
+	if (widget->parent != NULL && view->details->tree_widget == NULL) {
 		create_tree (view);
 		update_filtering_from_preferences (view);
 	}
@@ -1348,41 +1376,31 @@ filtering_changed_callback (gpointer callback_data)
 }
 
 static void
-load_location_callback (FMTreeView *view, char *location)
+loading_uri_callback (NautilusWindowInfo *window,
+		      char *location,
+		      FMTreeView *view)
 {
-	if (view->details->selection_location != NULL) {
-		g_free (view->details->selection_location);
-	}
-	view->details->selection_location = g_strdup (location);
-
-	schedule_show_selection (view);
+	schedule_select_and_show_location (view, location);
 }
 
 static void
-fm_tree_view_instance_init (FMTreeView *view)
+fm_tree_view_init (FMTreeView *view)
 {
-	BonoboControl *control;
-	
 	view->details = g_new0 (FMTreeViewDetails, 1);
 	
-	
-	view->details->scrolled_window = gtk_scrolled_window_new (NULL, NULL);
-	
-	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (view->details->scrolled_window), 
+	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (view), 
 					GTK_POLICY_AUTOMATIC,
 					GTK_POLICY_AUTOMATIC);
+	gtk_scrolled_window_set_hadjustment (GTK_SCROLLED_WINDOW (view), NULL);
+	gtk_scrolled_window_set_vadjustment (GTK_SCROLLED_WINDOW (view), NULL);
 	
-	gtk_widget_show (view->details->scrolled_window);
+	gtk_widget_show (GTK_WIDGET (view));
 
-	control = bonobo_control_new (view->details->scrolled_window);
-	g_signal_connect_object (control, "activate",
-				 G_CALLBACK (tree_activate_callback), view, 0);
-
-	nautilus_view_construct_from_bonobo_control (NAUTILUS_VIEW (view), control);
+	g_signal_connect_object (view, "parent_set",
+				 G_CALLBACK (parent_set_callback), view, 0);
 
 	view->details->selection_location = NULL;
-	g_signal_connect_object (view, "load_location",
-				 G_CALLBACK (load_location_callback), view, 0);
+	
 	view->details->selecting = FALSE;
 
 	eel_preferences_add_callback (NAUTILUS_PREFERENCES_SHOW_HIDDEN_FILES,
@@ -1391,13 +1409,6 @@ fm_tree_view_instance_init (FMTreeView *view)
 				      filtering_changed_callback, view);
 	eel_preferences_add_callback (NAUTILUS_PREFERENCES_TREE_SHOW_ONLY_DIRECTORIES,
 				      filtering_changed_callback, view);
-	g_signal_connect_object (view, "show_hidden_files_mode_changed",
-				 G_CALLBACK (filtering_changed_callback), view, 0);  
-
-	nautilus_view_set_listener_mask
-		(NAUTILUS_VIEW (view),
-		 NAUTILUS_VIEW_LISTEN_SHOW_HIDDEN_FILES_MODE);
-
 	
 	g_signal_connect_object (nautilus_icon_factory_get(), "icons_changed",
 				 G_CALLBACK (theme_changed_callback), view, 0);  
@@ -1465,3 +1476,107 @@ fm_tree_view_class_init (FMTreeViewClass *class)
 	
 	copied_files_atom = gdk_atom_intern ("x-special/gnome-copied-files", FALSE);
 }
+
+static const char *
+fm_tree_view_get_sidebar_id (NautilusSidebar *sidebar)
+{
+	return TREE_SIDEBAR_ID;
+}
+
+static char *
+fm_tree_view_get_tab_label (NautilusSidebar *sidebar)
+{
+	return g_strdup (_("Tree"));
+}
+
+static char *
+fm_tree_view_get_tab_tooltip (NautilusSidebar *sidebar)
+{
+	return g_strdup (_("Show Tree"));
+}
+
+static GdkPixbuf *
+fm_tree_view_get_tab_icon (NautilusSidebar *sidebar)
+{
+	return NULL;
+}
+
+static void
+fm_tree_view_is_visible_changed (NautilusSidebar *sidebar,
+				 gboolean         is_visible)
+{
+	/* Do nothing */
+}
+
+static void 
+hidden_files_mode_changed_callback (NautilusWindowInfo *window,
+				    FMTreeView *view)
+{
+	update_filtering_from_preferences (view);
+}
+
+static void
+fm_tree_view_iface_init (NautilusSidebarIface *iface)
+{
+	iface->get_sidebar_id = fm_tree_view_get_sidebar_id;
+	iface->get_tab_label = fm_tree_view_get_tab_label;
+	iface->get_tab_tooltip = fm_tree_view_get_tab_tooltip;
+	iface->get_tab_icon = fm_tree_view_get_tab_icon;
+	iface->is_visible_changed = fm_tree_view_is_visible_changed;
+}
+
+static void
+fm_tree_view_set_parent_window (FMTreeView *sidebar,
+				NautilusWindowInfo *window)
+{
+	char *location;
+	
+	sidebar->details->window = window;
+
+	g_signal_connect_object (window, "loading_uri",
+				 G_CALLBACK (loading_uri_callback), sidebar, 0);
+	location = nautilus_window_info_get_current_location (window);
+	loading_uri_callback (window, location, sidebar);
+	g_free (location);
+
+	g_signal_connect_object (window, "hidden_files_mode_changed",
+				 G_CALLBACK (hidden_files_mode_changed_callback), sidebar, 0);  
+
+}
+
+static NautilusSidebar *
+fm_tree_view_create (NautilusSidebarProvider *provider,
+		     NautilusWindowInfo *window)
+{
+	FMTreeView *sidebar;
+	
+	sidebar = g_object_new (fm_tree_view_get_type (), NULL);
+	fm_tree_view_set_parent_window (sidebar, window);
+	g_object_ref (sidebar);
+	gtk_object_sink (GTK_OBJECT (sidebar));
+
+	return NAUTILUS_SIDEBAR (sidebar);
+}
+
+static void 
+sidebar_provider_iface_init (NautilusSidebarProviderIface *iface)
+{
+	iface->create = fm_tree_view_create;
+}
+
+static void
+fm_tree_view_provider_init (FMTreeViewProvider *sidebar)
+{
+}
+
+static void
+fm_tree_view_provider_class_init (FMTreeViewProviderClass *class)
+{
+}
+
+void
+fm_tree_view_register (void)
+{
+        nautilus_module_add_type (fm_tree_view_provider_get_type ());
+}
+
