@@ -388,6 +388,12 @@ nautilus_notes_viewer_get_tab_label (NautilusSidebar *sidebar)
 	return g_strdup (_("Notes"));
 }
 
+static char *
+nautilus_notes_viewer_get_tab_tooltip (NautilusSidebar *sidebar)
+{
+	return g_strdup (_("Show Notes"));
+}
+
 static GdkPixbuf *
 nautilus_notes_viewer_get_tab_icon (NautilusSidebar *sidebar)
 {
@@ -415,6 +421,7 @@ nautilus_notes_viewer_sidebar_iface_init (NautilusSidebarIface *iface)
 {
 	iface->get_sidebar_id = nautilus_notes_viewer_get_sidebar_id;
 	iface->get_tab_label = nautilus_notes_viewer_get_tab_label;
+	iface->get_tab_tooltip = nautilus_notes_viewer_get_tab_tooltip;
 	iface->get_tab_icon = nautilus_notes_viewer_get_tab_icon;
 	iface->is_visible_changed = nautilus_notes_viewer_is_visible_changed;
 }
@@ -430,14 +437,9 @@ nautilus_notes_viewer_set_parent_window (NautilusNotesViewer *sidebar,
         sidebar->details->uri = nautilus_window_info_get_current_location (window);
         notes_load_metainfo (sidebar);
 
-        /* handle selections */
-#ifdef GNOME2_CONVERSION_COMPLETE
-        /* note_text_field is a text-view, not a GtkEditable */
-	nautilus_clipboard_set_up_editable
-                (GTK_EDITABLE (sidebar->details->note_text_field),
-		 nautilus_window_info_get_ui_manager (window),
-		 FALSE);
-#endif
+	nautilus_clipboard_set_up_text_view
+		(GTK_TEXT_VIEW (sidebar->details->note_text_field),
+		 nautilus_window_info_get_ui_manager (window));
 }
 
 static NautilusSidebar *
@@ -487,6 +489,7 @@ get_property_pages (NautilusPropertyPageProvider *provider,
         uri = nautilus_file_info_get_uri (file);
 	
 	viewer = g_object_new (nautilus_notes_viewer_get_type (), NULL);
+        g_free (viewer->details->uri);
         viewer->details->uri = uri;
         notes_load_metainfo (viewer);
 
